//=======================================================================
// DingMee Translator
// URLs: http://chrm.info/cms/dingmee-translator
// Copyright (C) 2012, Christoph Maciejewski
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#include "dictdownloader.h"

DictDownloader::DictDownloader(QDeclarativeItem *parent) :
    QDeclarativeItem(parent)
{
    qDebug() << "DictDownloader!";
    this->downloadID = 0;
    this->successfullyDownloaded = -1;
    ftp = (QFtp*)0;

    this->downloadIsActive = false;

    this->ftpUser = "";
    this->ftpPasswd = "";
}

void DictDownloader::startDownload()
{
    this->successfullyDownloaded = 0;
    this->downloadIsActive = true;

    if(this->downloadQueue.size() > 0) {
        // Next entry
        QStringList entry = this->downloadQueue.first().split("|");
        this->downloadQueue.removeAt(0);
        this->downloadDict(entry[0], entry[1], entry[2], false);
    }
}

bool DictDownloader::isDictInQueue(QString _name) {
    _name = _name.toLower();

    if(this->downloadName.toLower().contains(_name))
        return true;

    for(int i=0; i<this->downloadQueue.size(); i++) {
        if(this->downloadQueue.at(i).contains(_name))
            return true;
    }

    return false;
}

bool DictDownloader::isDownloadActive() {
    if(this->downloadQueue.size() > 0)
        return true;

    return this->downloadIsActive;
}

void DictDownloader::downloadDict(QString _localPath, QString _url, QString _downloadName, bool _newEntry)
{
    this->fileName = _localPath;
    this->ftpUrl = _url;    
    this->downloadName = _downloadName;

    if(_newEntry) {
        this->downloadQueue << (_localPath + "|" + _url + "|" + _downloadName);
        return;
    }

    this->file = new QFile(this->fileName+".tmp");

    if (!this->file->open(QIODevice::WriteOnly)) {
        qDebug() << "Error: could not open file: " << this->fileName;
        this->lastStatusMessage = "Error: could not open file:" + this->fileName;
        delete this->file;
        return;
    }

    QUrl url(this->ftpUrl);
    qDebug() << "Connect to host: " << url.encodedHost();
    lastStatusMessage = "Connect to host: " + url.encodedHost();

    if (ftp != (QFtp*)0) {
        ftp->abort();
        ftp->deleteLater();
        ftp = 0;
    }

    qDebug() << "Host: " << url.encodedHost();
    qDebug() << "Path: " << url.encodedPath();

    // Overwrite path, remove hostname
    this->ftpUrl = url.encodedPath();


    ftp = new QFtp(this);

    connect(ftp, SIGNAL(commandFinished(int,bool)),
            this, SLOT(ftpCommandFinished(int,bool)));
    connect(ftp, SIGNAL(stateChanged(int)),
            this, SLOT(ftpStateChanged(int)));
    connect(ftp, SIGNAL(commandStarted(int)),
            this, SLOT(ftpCommandStarted(int)));
    connect(ftp, SIGNAL(dataTransferProgress(qint64,qint64)),
            this, SLOT(updateDataTransferProgress(qint64,qint64)));

    this->currentProgress = "?/?";    
    ftp->connectToHost(url.encodedHost(), 21);
}

void DictDownloader::updateDataTransferProgress(qint64 _part, qint64 _complete)
{
    qDebug() << "Transfered: " << _part << " of " << _complete;
    this->currentProgress = (QString::number(_part/1000) + "/" + QString::number(_complete/1000));
    this->lastStatusMessage = "Downloading " + this->downloadName + ": " + this->currentProgress;
}

void DictDownloader::ftpCommandFinished(int _id, bool _error)
{
    if(_error) {
        this->lastStatusMessage = "Error: " + ftp->errorString() + " (" + this->ftpUrl + ")";
        qDebug() << "Error occures: " << ftp->errorString();
        return;
    }

    qDebug() << "FTP finished. ID: " << _id;

    if(_id == this->downloadID) {
        // Download was successfull
        // Rename file
        if(this->file == (QFile*)0)
            return;

        if (this->file->exists() && this->file->size() > 10000)
        {
            qDebug() << "FTP download finished, rename file...";

            QFile old(this->fileName);
            if(old.exists())
                old.remove();

            this->file->rename(this->fileName);
            this->file->close();
            this->successfullyDownloaded++;
            this->lastStatusMessage = this->downloadName + " successfully downloaded";

            if(this->downloadQueue.size() > 0) {
                // Next entry
                QStringList entry = this->downloadQueue.first().split("|");
                this->downloadQueue.removeAt(0);
                this->downloadDict(entry[0], entry[1], entry[2], false);
            } else {
                // All finished
                this->lastStatusMessage = this->successfullyDownloaded + " dicts downloaded.";
                this->downloadIsActive = false;
            }
        }
    }
}

void DictDownloader::ftpCommandStarted(int _id)
{
    qDebug() << "FTP started. ID:" << _id;
    this->lastStatusMessage = "FTP started. ID:" + QString::number(_id);
}

void DictDownloader::cancelDownload()
{
    if (ftp == (QFtp*)0) {
        return;
    }

    this->ftp->abort();
    this->downloadQueue.clear();
    this->downloadName.clear();
    this->downloadIsActive = false;

    this->lastStatusMessage = "Download cancelled";
    qDebug() << "Download cancelled";

    if(this->file == (QFile*)0)
        return;

    if (this->file->exists()) {
        this->file->close();
        this->file->remove();
    }

    delete this->file;
    this->file = (QFile*)0;
}

void DictDownloader::ftpStateChanged(int _state)
{
    qDebug() << "ftpStateChanged() " << _state;

    switch(_state)
    {
        case QFtp::Unconnected:
        {
            this->lastStatusMessage = "FTP: Disconnected";
            qDebug() << "FTP: Disconnected.";
        }
        break;
        case QFtp::HostLookup:
        {
            this->lastStatusMessage = "FTP: HostLookup";
            qDebug() << "FTP: HostLookup...";
        }
        break;
        case QFtp::Connecting:
        {
            this->lastStatusMessage = "FTP: Connecting";
            qDebug() << "FTP: Connecting...";
        }
        break;
        case QFtp::Connected:
        {
            this->lastStatusMessage = "FTP: Connected";
            qDebug() << "FTP: Connected. Login...";
            ftp->login(this->ftpUser, this->ftpPasswd);
        }
        break;
        case QFtp::Closing:
        {
            this->lastStatusMessage = "FTP: Closing";
            qDebug() << "FTP: Closing...";
        }
        break;
        case QFtp::LoggedIn:
        {
            this->lastStatusMessage = "FTP: LoggedIn. Getting file...";
            qDebug() << "FTP: LoggedIn. Getting file...";
            this->downloadID = ftp->get(this->ftpUrl, this->file);
            ftp->close();
        }
        break;
    }

}





