//=======================================================================
// DingMee Translator
// URLs: http://chrm.info/cms/dingmee-translator
// Copyright (C) 2012, Christoph Maciejewski
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#include "settingsmanager.h"
#include "QSettings"
#include <QFileInfo>
#include "dictdownloader.h"
#include "common.h"

SettingsManager::SettingsManager(QDeclarativeItem *parent)
    : QDeclarativeItem(parent)
{
    qDebug() << "SettingsManager!";

    this->dictNames << "de-en" << "en-es" << "en-pl" << "en-it" << "en-ru" << "en-fi" <<
                       "en-cmn" << "en-ja" << "en-fr" << "en-arb" << "en-cs" << "en-sv" <<
                       "en-el" << "en-he" << "en-pt" << "en-hu" << "es-de" << "de-pl" << "de-fr" <<
                       "de-it" << "ru-he" << "ru-pl" << "en-no" << "en-ro" << "en-tr" << "en-hi" <<
                       "de-tr" << "de-cmn" << "it-fr" << "es-fr" << "en-ko" << "en-vi";

    this->readSettings();

    // Check for old version
    if(this->release != RELEASE) {
        QSettings settings("chrm", "DingMee");
        settings.clear();
        this->readSettings();
    }

    this->writeSettings();
}

void SettingsManager::writeSettings()
{
    QSettings settings("chrm", "DingMee");
    settings.beginGroup("settings");

    int i=0;
    for(i=0; i<SUPPORTED_LANGUAGES; i++) {
        settings.setValue("dictpath_"+this->dictNames[i], this->dictionaryPaths[i]);
        settings.setValue("ftppath_"+this->dictNames[i], this->ftpPaths[i]);
    }

    settings.setValue("checkupdates", this->checkForUpdates);
    settings.setValue("release", RELEASE);
    settings.setValue("lastLanguage", this->lastLanguage);
    settings.endGroup();
}

QString SettingsManager::getLastLang()
{
    return this->lastLanguage;
}

void SettingsManager::readSettings()
{
    QSettings settings("chrm", "DingMee");
    QString dirPrefix = "/home/user/MyDocs/";

    // Win32 environment
    QFileInfo qfi("D:\\temp\\QtProjects\\dingmee\\");
    if(qfi.isDir()) {
        dirPrefix = "D:\\temp\\QtProjects\\dingmee\\";
        qDebug() << "Use: " + dirPrefix;
    }

    settings.beginGroup("settings");
    int i=0;
    for(i=0; i<SUPPORTED_LANGUAGES; i++) {
        this->dictionaryPaths[i] = settings.value("dictpath_"+this->dictNames[i], dirPrefix+this->dictNames[i]+".txt").toString();        
        this->ftpPaths[i] = settings.value("ftppath_"+this->dictNames[i], "ftp://chrm.info/"+this->dictNames[i]+".txt").toString();
    }
    this->checkForUpdates = settings.value("checkupdates", false).toBool();
    this->release = settings.value("release", "0.0.0").toString();
    this->lastLanguage = settings.value("lastLanguage", "de-en").toString();
    settings.endGroup();
}

QString SettingsManager::getDictPath(int _lang)
{
    return this->dictionaryPaths[_lang];
}

void SettingsManager::setDictPath(QString _path, int _lang)
{
    this->dictionaryPaths[_lang] = _path;
    this->writeSettings();
}

QString SettingsManager::getFTPPath(int _lang)
{
    return this->ftpPaths[_lang];
}

void SettingsManager::setFTPPath(QString _path, int _lang)
{
    this->ftpPaths[_lang] = _path;
    this->writeSettings();    
}

void SettingsManager::downloadDict(int _lang) {
    if(this->dd == NULL) {
        this->dd = new DictDownloader(this);
    }

    qDebug() << "Call downloadDict(" << _lang << ")";
    this->dd->downloadDict(this->dictionaryPaths[_lang], this->ftpPaths[_lang], this->dictNames[_lang]);
}

void SettingsManager::startDownload() {
    qDebug() << "Call startDownload()";

    if(this->dd == NULL) {
        return;
    }

    this->dd->startDownload();
}

QString SettingsManager::getStatus()
{
    if(this->dd == NULL) {
        return "---";
    }

    // DEBUG
    qDebug() << "GetStatus: " << this->dd->lastStatusMessage;

    return this->dd->lastStatusMessage;
}

void SettingsManager::cancelFtp()
{
    if(this->dd == NULL) {
        return;
    }
    this->dd->cancelDownload();
}

QString SettingsManager::getDictInfo()
{
    QString retVal = "<b><u>About local copy:</u><b><br><font size='-2'>";
    QString dirPrefix = "/home/user/MyDocs/";

    retVal += "Directory: " + dirPrefix + "<br>";

    for(int i=0; i<this->dictionaryPaths.size(); i++) {
        QFileInfo info1(this->dictionaryPaths[i]);        

        if(!info1.exists()) {
            //retVal += ("<b>" + this->dictNames[i] + ":</b> ");
            //retVal += "<font color='red'>Not installed. Download first.</font><br>";
        } else {            
            retVal += ("<b>" + this->dictNames[i] + ":</b> ");
            retVal += "&nbsp;Size: " + QString::number(info1.size()/1024) + " KiB" + " Modified: " + info1.lastModified().toString("dd.MM.yyyy") + "<br>";
        }
    }

    retVal += "</font>";
    return retVal;
}

bool SettingsManager::isDictAvailable()
{
    if(this->dd != NULL) {
        if(this->dd->isDownloadActive())
            return false;
    }

    for(int i=0; i<SUPPORTED_LANGUAGES; i++) {
        if(isDictAvailable(i))
                return true;
    }

    return false;
}

bool SettingsManager::isDictAvailable(int _dict)
{
    QFileInfo info1(this->dictionaryPaths[_dict]);

    if(info1.exists() && info1.size() > 100000)
        return true;

    if(this->dd != NULL) {
        if(this->dd->isDictInQueue(this->dictNames[_dict]))
            return true;
    }

    return false;
}

void SettingsManager::removeAllDicts()
{
    for(int i=0; i<this->dictionaryPaths.size(); i++) {
        if(this->isDictAvailable(i)) {
            QFile dict(this->dictionaryPaths[i]);
            if(dict.remove())
                qDebug() << this->dictionaryPaths[i] << " removed.";
            else
                qDebug() << this->dictionaryPaths[i] << ": can't remove.";

            QFile dictTmp(this->dictionaryPaths[i]+".tmp");
            dictTmp.remove();
        }
    }
}
